

estimate_covfunction <- function (sample_ids, output_direct, input_file_names, analysis_lab, min_readlength, max_fragmsize, monoProc_load ) {


#  ARGUMENTS:
# sample_ids       = vector of strings with sample IDs (used to label individual output files)
# output_direct    = directory where all output (=files + figures) will be written
# input_file_names = names of the files with the read count data
# analysis_lab     = label that characterizes the analysis and will be used to create the output files
# min_readlength   = minimum possible read length
# max_fragmsize    = estimate of maximum fragment size (that will have probabilities of zero)    


# SOME MORE OPTIONS
  plot_raw_fit            <- "yes" # create plots of read start counts and smootheed curves all individual samples 
  plot_coverage_functions <- "yes" # create plots of estimates coverage curves for all individual samples 
  aggregate_tail_data     <- "yes"  # if "yes"  replace fitted value for start (read_lenght -1) and end position (max_fragmsize+1) with the mean of multiple sites that should all have same expected value to improve precision 

if (monoProc_load=="no") library("monoProc", lib.loc="./") else library(monoProc)

  n_sample       <- length(sample_ids)
  n_methods      <- 2
  method_labels  <- c("kernel","q_spline")
  readstart_data <- read.csv( input_file_names[1] )
  region_size    <- length(readstart_data[,1]) - 1
  print(paste(" Constant d that defines the [C-d,C+d] interval used to select isolated CpGs was ",region_size,sep="") )    
  
if (aggregate_tail_data == "yes") { 
      end_prob_0   <- min(max_fragmsize+50,region_size)    
      start_prob_1 <- min_readlength - 5 }

  figfile     <- paste(output_direct,analysis_lab,"_figures.pdf",sep="")
  pdf(file = figfile )
  plot_min    <- min_readlength
  plot_max    <- max_fragmsize-1

  read_pos        <- 0:region_size
  max_range       <- region_size + 1
  n_fragm          <- max_fragmsize-min_readlength+1
  readstart_dist  <- rep(NA,(n_fragm-1))
  n_fragm_min1    <- n_fragm-1

  n_obs             <- matrix(NA,n_sample,1)
  all_data          <- rep(0,region_size +1) 
  frag_stand        <- matrix(NA,n_sample,n_methods) 
  rownames(n_obs)   <- sample_ids
  rownames(frag_stand) <- sample_ids
  colnames(frag_stand) <- method_labels

  kernel_cov_dist   <- cbind(min_readlength:(max_fragmsize-1), matrix(NA,n_fragm_min1 ,n_sample) )
  spline_cov_dist   <- cbind(min_readlength:(max_fragmsize-1), matrix(NA,n_fragm_min1 ,n_sample) )
  colnames(kernel_cov_dist )  <- c("pos",sample_ids)
  colnames(spline_cov_dist )  <- c("pos",sample_ids)
  select_fit_results  <- read_pos >= (min_readlength-1) & read_pos <= max_fragmsize
  fit_results  <- cbind(0:(max_range-1), matrix(NA,max_range,n_methods) )
  colnames(fit_results) <- c("pos",method_labels)
  frag_dist    <- cbind(min_readlength:max_fragmsize, matrix(NA,n_fragm,n_methods) ) ; colnames(frag_dist) <- colnames(fit_results)
  cov_dist     <- cbind(min_readlength:(max_fragmsize-1), matrix(NA,n_fragm_min1 ,n_methods) ) ; colnames(cov_dist) <- colnames(fit_results)
  index_i      <- 1:n_fragm_min1
  index_iplus1 <- 2:n_fragm
  correction   <- min_readlength:(max_fragmsize-1)
  pos_zero     <- max_fragmsize - min_readlength

  select  <- kernel_cov_dist[,"pos"] >= plot_min & kernel_cov_dist[,"pos"] <= plot_max
 
  i <- 1
  for (i in 1:(n_sample+1)) {
  
    print(paste("sample processed = ",sample_ids[i],sep=""))
    if (i <= n_sample) {

       readstart_data <- read.csv( input_file_names[i] )
       readstart_data <-  readstart_data[,2] 
 
        n_obs[i]       <- sum( readstart_data )
        all_data       <- all_data + (50000/n_obs[i]) * readstart_data } # standardize on read count so that every sample will have equal weight

    if (i == (n_sample+1) ) readstart_data <- all_data

    temp            <- ksmooth(read_pos,readstart_data,"normal",  x.points= read_pos, bandwidth=15)
    fit_results[,2] <- temp@y

    temp            <- smooth.spline(read_pos,readstart_data)
    fit_results[,3] <- temp$y

    if (plot_raw_fit=="yes") {
         if (i < (n_sample+1)) fig_title = paste("Read start counts and fit: ",sample_ids[i],sep="") else 
                    fig_title <- "Read counts and smoother fit for all samples added" 
         plot(read_pos,readstart_data,type="p",main=fig_title,ylab="Read counts",xlab="Positions")
         lines(read_pos,fit_results[,2],col="forestgreen",type="l")
         lines(read_pos,fit_results[,3],col="red",type="l")
         legend("topright", inset=.005,  method_labels, fill=c("forestgreen","red"), horiz=F)
    }

    j <- 1
    for (j in 1:n_methods) {

    temp <- fit_results[select_fit_results,j+1] 
    if (aggregate_tail_data == "yes") {
        temp[1]         <- mean(fit_results[start_prob_1:min_readlength,j+1] )
        temp[n_fragm+1] <- mean(fit_results[(max_fragmsize+1):end_prob_0,j+1] )
    }

# turn read to CpG data into proportions
    temp <- temp / sum(temp)    

# calculate fragment size distrbution
    frag_dist[,j+1]   <- c( (temp[index_i] - temp[index_iplus1])  * correction,
                            (temp[n_fragm] - temp[n_fragm+1] ) * max_fragmsize )

# calculate coverage distribution 
      for (k in 1:n_fragm_min1) cov_dist[k,j+1] <- sum( frag_dist[(k+1):n_fragm_min1,j+1] )


# additional step to make coverage function monotone descending
# adding postion at the begin and end because monoproc can behave poorly at first and last position 
# never use monoproc over a larger range as you will get when bias fitting it on a staight line (=being and end)
# ksmooth only really needed here to create object that can be used by monoproc: Hence ksmooth just draws line through points 
      temp1          <- c( min_readlength-1,cov_dist[,1],max_fragmsize)
      temp2          <- c( max(cov_dist[,j+1]), cov_dist[,j+1], min(cov_dist[,j+1]))
      temp           <- ksmooth( temp1, temp2,"normal", x.points=temp1,bandwidth=0.5)
      fit_ms         <- monoproc(temp,bandwidth=0.1,mono1="decreasing")
      temp           <- summary(fit_ms)
      temp@y         <- temp@y[-(n_fragm+1)]
      temp@y         <- temp@y[-1] 
      cov_dist[,j+1] <- temp@y

# make sure it ends at zero and starts at 1
      cov_dist[,j+1]      <- cov_dist[,j+1] - cov_dist[pos_zero,j+1] 
      cov_fragm_1         <- cov_dist[1,j+1]
      if (frag_dist[1,j+1]>0) desired_cov_fragm_1 <- 1-frag_dist[1,j+1] else desired_cov_fragm_1 <- 1
      cov_dist[,j+1]      <- cov_dist[,j+1] * desired_cov_fragm_1 / cov_fragm_1
   
    } # end for (j in 1:n_methods)...

  
    if (i <= n_sample) {
     kernel_cov_dist[,i+1] <- cov_dist[,2] 
     spline_cov_dist[,i+1] <- cov_dist[,3]
     
     # calculate expected coverage and then fragment size standardization factors
     for (j in 1:n_methods) {
  
        for (r in 1:max_fragm_size) {
          total <- 0
          for (s in min_readlength:n_fragm) if (s >= r) total <- total + frag_dist[s-min_readlength+1,j+1] / s
          readstart_dist[r] <- total
          }

        readstart_dist  <- readstart_dist / sum(readstart_dist)
        frag_stand[i,j] <-  1 / sum (readstart_dist * c(rep(1,min_readlength),cov_dist[,j+1]) )
  
   } # end for (j in 1:n_methods)...
  
  } # end if (i <= n_sample)

    if (plot_coverage_functions=="yes") { 
      if (i < (n_sample+1)) fig_title = paste("Coverage function for sample: ",sample_ids[i],sep="") else 
                    fig_title <- "Coverage function for all samples added" 
       j <- 1  
       plot( cov_dist[select,"pos"],cov_dist[select,j+1],type="l",col="forestgreen",main=fig_title,ylab="Read counts",xlab="Positions" )
       j <- 2  
       lines( cov_dist[select,"pos"],cov_dist[select,j+1],type="l",col="red" )
       legend("topright", inset=.005, method_labels , fill=c("forestgreen","red"), horiz=F)
     }
             
  } # for (i in 1:n_sample)

 
   names(n_obs) <- sample_ids
   write.csv(n_obs,paste(output_direct,analysis_lab,"_nobs.csv",sep=""),quote=F,row.names=T)

   temp <- cbind(0:(min_readlength-1),matrix(1,min_readlength,n_sample))
   write.csv(rbind( temp, kernel_cov_dist ),paste(output_direct,analysis_lab,"_kernel_cov_func.csv",sep=""),quote=F,row.names=F)
   write.csv(rbind( temp, spline_cov_dist ),paste(output_direct,analysis_lab,"_spline_cov_func.csv",sep=""),quote=F,row.names=F)
 
   sum_data_cov <- cbind(kernel_cov_dist[,1],
                       rowMeans(kernel_cov_dist[,-1]),
                       rowMeans(spline_cov_dist[,-1]),
                       cov_dist[,-1] )
   colnames(sum_data_cov) <- c("pos", paste("mean_",method_labels,sep=""),paste("allData_",method_labels,sep=""))

   temp <- cbind( 0:(min_readlength-1), matrix(1,min_readlength,2*n_methods) )
   write.csv(rbind( temp, sum_data_cov),paste(output_direct,analysis_lab,"_aggregate_cov_func.csv",sep=""),quote=F,row.names=F)  

   write.csv(frag_stand[,1],paste(output_direct,analysis_lab,"_kernel_stand_factors.csv",sep=""),quote=F,row.names=T)
   write.csv(frag_stand[,2],paste(output_direct,analysis_lab,"_spline_stand_factors.csv",sep=""),quote=F,row.names=T)

   sum_data_stand <- c(mean(frag_stand[,1]),
                       mean(frag_stand[,2]),
                           NA,
                           NA)
   # calculate expected coverage for all_data
     for (j in 1:n_methods) {
  
        for (r in 1:max_fragm_size) {
          total <- 0
          for (s in min_readlength:n_fragm) if (s >= r) total <- total + frag_dist[s-min_readlength+1,j+1] / s
          readstart_dist[r] <- total
          }

        readstart_dist  <- readstart_dist / sum(readstart_dist)
        sum_data_stand[n_methods+j] <- 1 / sum (readstart_dist * c(rep(1,min_readlength),cov_dist[,j+1]) )
  
   } # end for (j in 1:n_methods)...

   names(sum_data_stand) <- c(paste("mean_",method_labels,sep=""),paste("all_data_",method_labels,sep=""))
   write.csv(sum_data_stand,paste(output_direct,analysis_lab,"_aggregate_stand_factors.csv",sep=""),quote=F,row.names=T)

  

# create summary plots

    hist(n_obs,main="Histogram of read start position counts",xlab="Number of reads per sample",ylab="Number of samples")

    select  <- cov_dist[,"pos"] >= plot_min & cov_dist[,"pos"] <= plot_max
    j <- 1
    plot( sum_data_cov[select,"pos"],sum_data_cov[select,j+1],type="l",col="black",main="Coverage functions across all samples",xlab="Position",ylab="Coverage")
    j <- 2
    lines(sum_data_cov[select,"pos"],sum_data_cov[select,j+1],type="l",col="blue")
    j <- 3
    lines(sum_data_cov[select,"pos"],sum_data_cov[select,j+1],type="l",col="forestgreen")
    j <- 4
    lines(sum_data_cov[select,"pos"],sum_data_cov[select,j+1],type="l",col="red")
    legend("topright", inset=.005,  c(paste("mean_",method_labels,sep=""),paste("allData_",method_labels,sep="")) , fill=c("black","blue","forestgreen","red"), horiz=F)

 
    dev.off()

} # end est_covfunction <- function






