### R code from vignette source 'monoproc.snw'
### Encoding: UTF-8

###################################################
### code chunk number 1: monoproc.snw:61-62
###################################################
options(width=65)


###################################################
### code chunk number 2: monoproc.snw:65-69
###################################################
library(monoProc)
data(cars)  
speed<-cars$speed
dist<-cars$dist


###################################################
### code chunk number 3: monoproc.snw:73-77
###################################################
#Comparison with R-function isoreg
fit1<-ksmooth(speed, dist, "normal", bandwidth=2.5)
fit2<-monoproc(fit1,bandwidth=0.7)
fit3<-isoreg(speed,dist)


###################################################
### code chunk number 4: a
###################################################
plot(fit3,plot.type="single", main="monotone regression", xlab="speed", ylab="distance",  par.fit = list(col = "black", cex = 1.5, pch = 13, lwd = 1.5, lty=3))
lines(fit1, lty=1, lwd=1.5)
lines(fit2, lty=2, lwd=1.5)
legend(5,100, c("isoreg","ksmooth","monoproc"),col=c(1,1,1), lty=c(3,1,2))


###################################################
### code chunk number 5: monoproc.snw:92-96
###################################################
cars.lo <- loess(dist ~ speed, cars, degree=2,
control = loess.control(surface = "direct"))
predict<-predict(cars.lo, data.frame(speed = seq(5, 25, 1)))
monofit<-monoproc(cars.lo, bandwidth=0.4, xx=seq(5,25,1),mono1="increasing", gridsize=30)


###################################################
### code chunk number 6: b
###################################################
plot(cars.lo, xlab="speed", ylab="dist")
lines(seq(5, 25, 1),predict)
lines(monofit,lty=2)
legend(5,100, c("loess","monoproc"), lty=c(1,2))


###################################################
### code chunk number 7: monoproc.snw:112-116
###################################################
d <- seq(0, 25, len = 200)
pr <- lm(dist ~ poly(speed, 4))
prlist<-list(speed=d,dist=predict(pr, data.frame(speed=d)))
mpr<-monoproc(prlist, bandwidth=0.05, mono1="increasing")


###################################################
### code chunk number 8: c
###################################################
plot(cars, xlab = "Speed (mph)", ylab = "Stopping distance (ft)",
         las = 1, xlim = c(0, 25))
lines(d, predict(pr, data.frame(speed=d)))
lines(mpr, lty=2)
legend(5,110, c("polynomial regression","monotonized polynom"), lty=c(1,2))


###################################################
### code chunk number 9: monoproc.snw:141-149
###################################################
library(UsingR)
library(locfit)
data(fat)
fat<-fat[-39,] ##two extreme observations
fat<-fat[-41,] ##are deleted
attach(fat)
fit<-locfit.raw(cbind(weight, height),body.fat.siri, alpha=0.3, deg=1, kern="epan")
fitmono<-monoproc(fit,bandwidth=1, mono1="increasing", mono2="decreasing",dir="xy", gridsize=30)


###################################################
### code chunk number 10: d
###################################################
plot(fit, type="persp", theta = 300, phi = 30,col="lightblue", cex=1.3, main="unconstrained Bodyfat estimate")


###################################################
### code chunk number 11: e
###################################################
plot(fit, main="contour plot of the unconstrained estimate")


###################################################
### code chunk number 12: f
###################################################
plot(fitmono,theta = 300, phi = 30, col="lightblue",cex=1.3, main="monotone Bodyfat estimate")


###################################################
### code chunk number 13: g
###################################################
plot(fitmono, type="contour", main="contour plot of the monotone estimate")


###################################################
### code chunk number 14: monoproc.snw:176-178
###################################################
t<-cv(fitmono)
sum((t[,2]-body.fat.siri)^2)/250    


###################################################
### code chunk number 15: monoproc.snw:181-182
###################################################
sum((t[,1]-body.fat.siri)^2)/250   #Cross Validation for the monotone estimator


###################################################
### code chunk number 16: monoproc.snw:191-201
###################################################
x<-seq(1:50)/51
y<-seq(1:70)/71
z<-matrix(0, nrow=50, ncol=70)
for(i in 1:70){
	for(j in 1:50){
		z[j,i]<-0.5*(y[i]+1/(3*pi)*sin(6*pi*y[i]))*(1+(2*x[j]-1)^3)
	}
}
list<-list(x=x, y=y, z=z)
mono<-monoproc(list, bandwidth=0.0009, dir="y", mono2="increasing")


###################################################
### code chunk number 17: h
###################################################
persp(x,y,z, phi=30, theta=250, col="lightblue", main="f(x,y)")


###################################################
### code chunk number 18: i
###################################################
plot(mono, phi=30, theta=250, col="lightblue", main="monotone f(x,y)")


###################################################
### code chunk number 19: j
###################################################
contour(x,y,z, main="contour plot f(x,y)")


###################################################
### code chunk number 20: k
###################################################
plot(mono,type="contour", main="contour plot monotone f(x,y)")


###################################################
### code chunk number 21: l
###################################################
plot(y,z[3,], type="l", main="fixed x=3/51")
lines(mono@fit@y,mono@fit@z[3,], lty=2)
legend(0.2,0.14, c("f(x,y)","monotonized f(x,y)"), lty=c(1,2))


###################################################
### code chunk number 22: m
###################################################
plot(x,z[,3], type="l", main="fixed y=3/101")
lines(mono@fit@x,mono@fit@z[,3], lty=2)
legend(0.2,0.10, c("f(x,y)","monotonized f(x,y)"), lty=c(1,2))


###################################################
### code chunk number 23: monoproc.snw:246-249
###################################################
slotNames("monoproc")
slotNames("monoproc.1d")
slotNames("monoproc.2d")


###################################################
### code chunk number 24: monoproc.snw:256-257
###################################################
monoproc(fit)


###################################################
### code chunk number 25: monoproc.snw:289-290
###################################################
plot(fit3,plot.type="single", main="monotone regression", xlab="speed", ylab="distance",  par.fit = list(col = "black", cex = 1.5, pch = 13, lwd = 1.5, lty=3))
lines(fit1, lty=1, lwd=1.5)
lines(fit2, lty=2, lwd=1.5)
legend(5,100, c("isoreg","ksmooth","monoproc"),col=c(1,1,1), lty=c(3,1,2))


###################################################
### code chunk number 26: monoproc.snw:294-295
###################################################
plot(cars.lo, xlab="speed", ylab="dist")
lines(seq(5, 25, 1),predict)
lines(monofit,lty=2)
legend(5,100, c("loess","monoproc"), lty=c(1,2))


###################################################
### code chunk number 27: monoproc.snw:299-300
###################################################
plot(cars, xlab = "Speed (mph)", ylab = "Stopping distance (ft)",
         las = 1, xlim = c(0, 25))
lines(d, predict(pr, data.frame(speed=d)))
lines(mpr, lty=2)
legend(5,110, c("polynomial regression","monotonized polynom"), lty=c(1,2))


###################################################
### code chunk number 28: monoproc.snw:304-308
###################################################
plot(fit, type="persp", theta = 300, phi = 30,col="lightblue", cex=1.3, main="unconstrained Bodyfat estimate")
plot(fit, main="contour plot of the unconstrained estimate")
plot(fitmono,theta = 300, phi = 30, col="lightblue",cex=1.3, main="monotone Bodyfat estimate")
plot(fitmono, type="contour", main="contour plot of the monotone estimate")


###################################################
### code chunk number 29: monoproc.snw:312-316
###################################################
persp(x,y,z, phi=30, theta=250, col="lightblue", main="f(x,y)")
plot(mono, phi=30, theta=250, col="lightblue", main="monotone f(x,y)")
contour(x,y,z, main="contour plot f(x,y)")
plot(mono,type="contour", main="contour plot monotone f(x,y)")


###################################################
### code chunk number 30: monoproc.snw:320-322
###################################################
plot(y,z[3,], type="l", main="fixed x=3/51")
lines(mono@fit@y,mono@fit@z[3,], lty=2)
legend(0.2,0.14, c("f(x,y)","monotonized f(x,y)"), lty=c(1,2))
plot(x,z[,3], type="l", main="fixed y=3/101")
lines(mono@fit@x,mono@fit@z[,3], lty=2)
legend(0.2,0.10, c("f(x,y)","monotonized f(x,y)"), lty=c(1,2))


