#include <cstdlib>
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <vector>
#include <cmath>
#include <stdio.h>
#include <time.h>

using namespace std;

const int max_coord         = 2451882;  // this is max cpg coord per chromosome: reset to larger number if needed
const int max_reads         = 20000000; // this is max reads per chromosome: reset to larger number if needed


int  cpg_coord_pos[max_coord], cpg_coord_neg[max_coord], sba_read_coord[max_reads], multi_read_coord[max_reads];
double sba_read_pos[max_reads], sba_read_neg[max_reads], multi_read_pos[max_reads], multi_read_neg[max_reads];

int  n_cpgs=0, n_multi_records = 0, n_sba_records=0;
double n_sba_reads = 0.0, n_multi_reads = 0.0; 

void read_cpgs(const char * filename);
void read_sba_reads(const char * filename);
void read_multi_reads(const char * filename);

void clean_sba_duplicate_peaks(int duplicate_hardthres, int duplicate_thres,int duplicate_neigh, int& n_sba_duplicate_peaks, double& n_sba_duplicate_reads );
void clean_multi_duplicate_peaks(int duplicate_hardthres, int duplicate_thres,int duplicate_neigh, int& n_multi_duplicate_peaks, double& n_multi_duplicate_reads);
int i_read=0,read_start=0,n_pileup_peaks = 0, neigh_count = 0, ichrom; 
double read_count=0.0; 
string strand;
ofstream outfile_peaks;



int main(int argc, char *argv[])
{

    // input 
    string sample_name = argv[1];       // sample ID
    string readdir     = argv[2];       // root directory where the reads are: assume sba and multi subdirectories  
    string cpgdir      = argv[3];       // isolated CpG coordinate file prefix to which program will add the string "chr#"
  
    // output
    string outdir         =  argv[4];   // output directory where the tables with counts of read start positions will be written
    string duplreadsdir   =  argv[5];   // output direcotory where the reads that were QC'ed out will be written
     
    // settings
    int n_chromosmomes = atoi(argv[6]); // number of chromosomes
 
    string multi_read  = argv[7];       // include multi reads (yes/no)
 
    int dupl_hardthres = atoi(argv[8]); // hard threshold: duplicate reads with counts higher than this will be eliminated
    int dupl_softthres = atoi(argv[9]); // soft threshold: for duplicate reads with counts higher than we will look in the neighboring to see if other read are aligning there
    int dupl_neigh     = atoi(argv[10]); // we will look +/- duplicate_neigh for other reads

    int  range      = atoi(argv[11]);    // this is the value d to define the [C-d,C+d] interval

  // default setting, file names and paths  
 
    string sba            = "yes";     // use the reads that are the single best alignments 
    string sba_readdir    = "sba/";    // subdirectory of readdir containing the .read files with the reads that are single best alignments
    string multi_readdir  = "multi/";  // subdirectory of readdir containing the .read files with the reads that are multi reads
 
  

//int main()
//{
//string sample_name;
//if (cluster == 0) sample_name = "01C12SM142AN";
//   else sample_name = "solid0558_FC2_20101109_0001";

    cout << " Sample being processed is " << sample_name << "\n";   
    cout << " \n";  
    cout << " Range d that that defines the [C-d,C+d] interval used to select isolated CpGs " << range << "\n";  
    cout << " \n";  
  
    
    int i, j, iter;
    string STRING, subst, filna;
	size_t pos, oldpos;
    stringstream ss;
	const char *ptemp;

    int i_cpg = 0, read2cp_dist=0;
    int min_dupl_thres = min(dupl_hardthres ,dupl_softthres); 
    double n_sba_dupl_reads=0.0, n_multi_dupl_reads=0.0; 
    int n_sba_dupl_peaks = 0, n_multi_dupl_peaks = 0; 
    int total_sba_records=0,total_multi_records=0,total_iso_sba_records=0,total_iso_multi_records=0;
    double total_sba_reads=0.0, total_multi_reads=0.0,total_iso_sba_reads=0.0, total_iso_multi_reads=0.0;

    struct tm *current;
	time_t now;
    time(&now);
	current = localtime(&now);
    cout << " Start at " << current->tm_hour << ":" << current->tm_min << ":" << current->tm_sec << "\n";
    cout << " \n";  

    if (sba=="yes") cout << " SBA reads included" << endl; 
       else cout << " SBA reads NOT included" << endl;
    if (multi_read=="yes") cout << " Multi-reads included" << endl; 
        else cout << " Multi-reads NOT included" << endl;
     cout << " \n";  
     
     
    // initialize table for read counts 
    double count_table[range];
    for(i=0;i<=range;i++) count_table[i] = 0;
    
  
    // open file to write qc'ed duplicate reads
    ss << duplreadsdir << sample_name << "_isoCpG_hard" << dupl_hardthres << "_soft" << dupl_softthres <<  "_neigh" << dupl_neigh  << ".csv"; 
    filna = ss.str(); ss.str(string()); ptemp= filna.c_str(); outfile_peaks.open(ptemp);  
    if (!(outfile_peaks).is_open()) {cout << " Output file with QC'ed duplicate reads " << ptemp << "  is not open \n"; exit (1);} else cout << " Output file with QC'ed duplicate reads " << ptemp << "\n";
    outfile_peaks << "chr,coord,pos,neg,readtype" << "\n"; // Creating header
        
  for (ichrom=1; ichrom<=n_chromosmomes; ++ichrom) {
    
    cout << "\n";   
    cout << " The chromosome being processed is " << ichrom << "\n"; 

   	// read the cpg coordinates //
    ss << cpgdir << "isoCpG_" << ichrom;
    filna = ss.str(); ss.str(string()); ptemp= filna.c_str();   
    read_cpgs(ptemp);  
    cout << "    number of isolated CpGs is " << n_cpgs << "\n";
    int n_cpgs_min1 =   n_cpgs - 1;

if (sba == "yes") {
    // read the sba read file //
    ss << readdir << sba_readdir << "chr" << ichrom  << "/" << sample_name << ".read";
    filna = ss.str(); ss.str(string()); ptemp= filna.c_str();   
    read_sba_reads(ptemp);  
    cout << "    the total number of unique SBA locations " << n_sba_records << "\n";      
    cout << "    the total number of SBA reads " << n_sba_reads << "\n";      
}
 
  
if (multi_read == "yes") { 
    // read the multi read file //
    ss << readdir << multi_readdir << "chr" << ichrom << "/" << sample_name << ".read";
    filna = ss.str(); ss.str(string()); ptemp= filna.c_str();   
    read_multi_reads(ptemp);  
    cout << "    the total number of unique multiread locations " << n_multi_records << "\n";      
    cout << "    the total number of (weighted) multireads " << n_multi_reads << "\n";      
}


//    total_sba_records = total_sba_records + n_sba_records;
//    total_sba_reads   = total_sba_reads   + n_sba_reads;

//    total_multi_records = total_multi_records + n_multi_records;
//    total_multi_reads   = total_multi_reads  + n_multi_reads;
    
      
if (sba == "yes") {
    // look for read starts on postive strand for SBA reads
    strand ="pos";
    i_cpg = 0;
    
    for (i_read=0; i_read<n_sba_records; ++i_read) {
        
       read_start = sba_read_coord[i_read];
       while ( cpg_coord_pos[i_cpg]<read_start && i_cpg<(n_cpgs-1) ) i_cpg=i_cpg+1; //find 1st cpg after read
     
       read2cp_dist = cpg_coord_pos[i_cpg]-read_start;

       if ( i_cpg == n_cpgs_min1 & read2cp_dist < 0 ) break;
      
       read_count = sba_read_pos[i_read] ;   
       if (read2cp_dist <= range & read_count > 0 ) {
  
          total_iso_sba_records = total_iso_sba_records + 1;
          total_iso_sba_reads   = total_iso_sba_reads   + read_count ;

          if (read_count> min_dupl_thres) clean_sba_duplicate_peaks(dupl_hardthres,dupl_softthres,dupl_neigh,n_sba_dupl_peaks,n_sba_dupl_reads );
          count_table[read2cp_dist] = count_table[read2cp_dist] + read_count;
          
          }  
   
    } // end for (i_read=0; i_read<n_sba_records; ++i_read)


  // negative strand  for SBA reads
    strand ="neg";
    i_cpg = n_cpgs-1;  
     for (i_read=(n_sba_records-1); i_read>=0; --i_read) {

        read_start = sba_read_coord[i_read];
        while ( cpg_coord_neg[i_cpg]>read_start && i_cpg>0 ) i_cpg=i_cpg-1; //find 1st cpg before read

        read2cp_dist = read_start-cpg_coord_neg[i_cpg];

       if ( i_cpg == 0 & read2cp_dist < 0 ) break;
                       
       read_count = sba_read_neg[i_read] ;   
       if (read2cp_dist <= range & read_count > 0 ) {

          total_iso_sba_records = total_iso_sba_records + 1;
          total_iso_sba_reads   = total_iso_sba_reads   + read_count ;

          if (read_count> min_dupl_thres) clean_sba_duplicate_peaks(dupl_hardthres,dupl_softthres,dupl_neigh, n_sba_dupl_peaks, n_sba_dupl_reads  );
          count_table[read2cp_dist] = count_table[read2cp_dist] + read_count;
        }  
    
    } // end  for (i_read=(n_sba_records)-1; i_read>=0; --i_read)

} // end SBA=1 


if (multi_read == "yes") {
    // look for read starts on postive strand for multi_read reads
    strand ="pos";
    i_cpg = 0;
    for (i_read=0; i_read<n_multi_records; ++i_read) {
    
       read_start = multi_read_coord[i_read];
       while ( cpg_coord_pos[i_cpg]<read_start && i_cpg<(n_cpgs-1) ) i_cpg=i_cpg+1; //find 1st cpg after read

       read2cp_dist = cpg_coord_pos[i_cpg]-read_start;
 
       if ( i_cpg == n_cpgs_min1 & read2cp_dist < 0 ) break; 
       
       read_count = multi_read_pos[i_read] ;   
       if (read2cp_dist <= range & read_count > 0 ) {

          total_iso_multi_records = total_iso_multi_records  + 1;
          total_iso_multi_reads   = total_iso_multi_reads  + read_count ;
 
          if (read_count> min_dupl_thres) clean_multi_duplicate_peaks(dupl_hardthres,dupl_softthres,dupl_neigh, n_multi_dupl_peaks, n_multi_dupl_reads  );
          count_table[read2cp_dist] = count_table[read2cp_dist] + read_count;
          }  
    
    } // end for (i_read=0; i_read<n_multi_read_records; ++i_read)

  // negative strand  for multi_read reads
    strand ="neg";
    i_cpg = n_cpgs-1;  
     for (i_read=(n_multi_records-1); i_read>=0; --i_read) {

        read_start = multi_read_coord[i_read];
        while ( (cpg_coord_neg[i_cpg]+1)>read_start && i_cpg>0 ) i_cpg=i_cpg-1; //find 1st cpg before read
   
        read2cp_dist = read_start-cpg_coord_neg[i_cpg];

         if ( i_cpg == 0 & read2cp_dist < 0 ) break;   
       
         read_count = multi_read_neg[i_read] ;   
         if (read2cp_dist <= range & read_count > 0 ) {
    
            total_iso_multi_records = total_iso_multi_records + 1;
            total_iso_multi_reads   = total_iso_multi_reads  + read_count ;
  
            if (read_count> min_dupl_thres) clean_multi_duplicate_peaks(dupl_hardthres,dupl_softthres,dupl_neigh, n_multi_dupl_peaks, n_multi_dupl_reads  );
            count_table[read2cp_dist] = count_table[read2cp_dist] + read_count;
       }  
      
    } // end  for (i_read=(n_multi_read_records)-1; i_read>=0; --i_read)

 }  // end multi = 1
 
 
 
} // end for (ichrom=1; ichrom<26; ++ichrom) 
 
   // Write read start table
    ofstream outfile;
 
    cout << " \n";  
    ss << outdir << sample_name << "_read2cpg_hard" << dupl_hardthres << "_soft" << dupl_softthres <<  "_neigh" << dupl_neigh  << ".csv";  
    filna = ss.str(); ss.str(string()); ptemp= filna.c_str(); outfile.open(ptemp); outfile.setf(ios::scientific); outfile.precision(15); 
    if (!(outfile).is_open()) {cout << " read2cpg outfile file " << ptemp << "  is not open \n"; exit (1);} else cout << " Read2cpg output file is " << ptemp << "\n";
    outfile << "pos,count" << "\n"; // Creating header
    for(i=0;i<=range;i++) outfile << i << "," << count_table[i] << endl;       
    outfile.close();
 
       
   cout << " \n";  
   ss << duplreadsdir << "summary_" <<  sample_name << "_isoCpG_hard" << dupl_hardthres << "_soft" << dupl_softthres <<  "_neigh" << dupl_neigh  << ".csv"; 
   filna = ss.str(); ss.str(string()); ptemp= filna.c_str(); outfile.open(ptemp); outfile.setf(ios::scientific); outfile.precision(15); 
   if (!(outfile).is_open()) {cout << " QC summary file named " << ptemp << "  is not open \n"; exit (1);} else cout << " QC summary file is " << ptemp << "\n";
   outfile << "iso_sba_records,"   << total_iso_sba_records << "\n"; 
   outfile << "iso_sba_reads,"     << total_iso_sba_reads << "\n"; 
   outfile << "sba_dupl_peaks,"    << n_sba_dupl_peaks << "\n"; 
   outfile << "sba_dupl_reads,"    << n_sba_dupl_reads  << "\n"; 

   outfile << "iso_multi_records," << total_iso_multi_records << "\n"; 
   outfile << "iso_multi_reads,"   << total_iso_multi_reads << "\n"; 
   outfile << "multi_dupl_peaks,"  << n_multi_dupl_peaks << "\n"; 
   outfile << "multi_dupl_reads,"  << n_multi_dupl_reads  << "\n"; 
   outfile.close();
  
 
    
  time(&now);
  current = localtime(&now);
  cout << " \n";  
  cout << " Finish at " << current->tm_hour << ":" << current->tm_min << ":" << current->tm_sec << "\n";

     return EXIT_SUCCESS;

}




//---------functions----------------
void read_cpgs(const char * filename) {

    string STRING;
	ifstream infile;
	int coord;

 	infile.open (filename);
    if (!(infile).is_open()) { cout << " CpG file " << filename << " is not open \n"; exit (1); }

    int i = 0;
    while (infile >> coord) {
        cpg_coord_pos[i] = coord; 
        cpg_coord_neg[i] = coord+1; 
        i++;
    }
    
    n_cpgs = i;

    infile.close();     
}     




void clean_sba_duplicate_peaks(int duplicate_hardthres, int duplicate_thres,int duplicate_neigh, int& n_sba_duplicate_peaks, double& n_sba_duplicate_reads) {
   int j=0;

   if (read_count > duplicate_hardthres) {
         n_sba_duplicate_peaks++;
         n_sba_duplicate_reads =  n_sba_duplicate_reads + (read_count-1);
         read_count = 1;
         outfile_peaks << ichrom << "," << read_start << "," << sba_read_pos[i_read] << "," <<  sba_read_neg[i_read] << "," << "1\n";
         return;
       }

// cout   << "test " << read_start << "," << sba_read_pos[i_read] << "," <<  sba_read_neg[i_read] << "\n";

   if (read_count > duplicate_thres) {

      neigh_count = 0;
//      cout   << "neigh_count a " << neigh_count << strand << "\n";
      
      if (strand=="pos") {if ( sba_read_neg[i_read] > 0 && sba_read_neg[i_read] <= duplicate_thres) neigh_count = 1; }// looks if something is in the other strand
      if (strand=="neg") {if ( sba_read_pos[i_read] > 0 && sba_read_pos[i_read] <= duplicate_thres) neigh_count = 1; }// looks if something is in the other strand

// cout   << "neigh_count b " << neigh_count << "\n";

      j = i_read-1;
      while  (  read_start - sba_read_coord[j] <= duplicate_neigh ) {    // look for other reads to the left     
         if ( sba_read_pos[j] > 0 && sba_read_pos[j] <= duplicate_thres  || 
              sba_read_neg[j] > 0 && sba_read_neg[j] <= duplicate_thres  ) {neigh_count =1; break;}
         if (j>0) j = j -1; else break;    }
// cout   << "neigh_count c " << neigh_count << "\n";

       j = i_read+1;
       while  (  sba_read_coord[j] - read_start <= duplicate_neigh ) {    // look for other reads to the right     
          if ( sba_read_pos[j] > 0 && sba_read_pos[j] <= duplicate_thres  || 
               sba_read_neg[j] > 0 && sba_read_neg[j] <= duplicate_thres  ) {neigh_count =1; break;}
          if (j < (n_sba_records-1)) j = j+1; else break;    }
//   cout   << "neigh_count d " << neigh_count << "\n";
 
          if (neigh_count < 1) {
                n_sba_duplicate_peaks++;
                n_sba_duplicate_reads = n_sba_duplicate_reads + (read_count-1);
                read_count = 1;
                neigh_count =0;
                outfile_peaks << ichrom << "," << read_start << "," << sba_read_pos[i_read] << "," <<  sba_read_neg[i_read] << "," << "1\n";
//   cout   << "neigh_count d " << read_count << "\n";

 
           }

   } // end if (read_count > duplicate_thres)
} // end void clean_duplicate_peaks() 



void clean_multi_duplicate_peaks(int duplicate_hardthres, int duplicate_thres,int duplicate_neigh, int& n_multi_duplicate_peaks, double& n_multi_duplicate_reads) {
   int j=0;

   if (read_count > duplicate_hardthres) {
         n_multi_duplicate_peaks++;
         n_multi_duplicate_reads = n_multi_duplicate_reads + (read_count-1);
         read_count = 1;
         outfile_peaks << ichrom << "," << read_start << "," << multi_read_pos[i_read] << "," <<  multi_read_neg[i_read] << "," << "2\n";
         return;
       }

   if (read_count > duplicate_thres) {

      neigh_count = 0;
      if (strand=="pos") {if ( multi_read_neg[i_read] > 0 && multi_read_neg[i_read] <= duplicate_thres) neigh_count = 1; }// looks if something is in the other strand
      if (strand=="neg") {if ( multi_read_pos[i_read] > 0 && multi_read_pos[i_read] <= duplicate_thres) neigh_count = 1; }// looks if something is in the other strand


      j = i_read-1;
      while  (  read_start - multi_read_coord[j] <= duplicate_neigh ) {    // look for other reads to the left     
         if ( multi_read_pos[j] > 0 && multi_read_pos[j] <= duplicate_thres  || 
              multi_read_neg[j] > 0 && multi_read_neg[j] <= duplicate_thres  ) {neigh_count =1; break;}
         if (j>0) j = j -1; else break;    }

       j = i_read+1;
       while  (  multi_read_coord[j] - read_start <= duplicate_neigh ) {    // look for other reads to the right     
          if ( multi_read_pos[j] > 0 && multi_read_pos[j] <= duplicate_thres  || 
               multi_read_neg[j] > 0 && multi_read_neg[j] <= duplicate_thres  ) {neigh_count =1; break;}
          if (j < (n_multi_records-1)) j = j+1; else break;    }
  
          if (neigh_count < 1) {
                n_multi_duplicate_peaks++;
                n_multi_duplicate_reads = n_multi_duplicate_reads + (read_count-1);
                read_count = 1;
                neigh_count =0;
                outfile_peaks << ichrom << "," << read_start << "," << multi_read_pos[i_read] << "," <<  multi_read_neg[i_read] << "," << "1\n";
           }

   } // end if (read_count > duplicate_thres)
} // end void clean_duplicate_peaks() 

void read_sba_reads(const char * filename) {

    string STRING, subst;
	ifstream infile;
	size_t pos, oldpos ;
	int i=0;
    int coordinate=0, prev_coordinate=0 ;
    double pos_count=0.0, neg_count=0.0, prev_pos_count=0.0, prev_neg_count=0.0 ;
    n_sba_reads =0;

	infile.open (filename);
    if (!(infile).is_open()) { cout << " Read file " << filename << " is not open \n"; exit (1); }
   	
    getline(infile,STRING); 	// skip header 
      	
    getline(infile,STRING);     // read first line
    oldpos = 0; pos=STRING.find(','); subst = STRING.substr(oldpos,pos-oldpos);
    prev_coordinate = atoi(subst.c_str());   

    oldpos = pos; pos=STRING.find(',',oldpos+1); subst = STRING.substr(oldpos+1,pos-oldpos-1);
    prev_pos_count = atof(subst.c_str());

    oldpos = pos; pos=STRING.find(',',oldpos+1); subst = STRING.substr(oldpos+1,pos-oldpos-1);
    prev_neg_count = atof(subst.c_str());    
    	      
	while(!infile.eof()) {            
  
        getline(infile,STRING); 
        if (STRING=="") break;
 
        oldpos = 0; pos=STRING.find(','); subst = STRING.substr(oldpos,pos-oldpos);
        coordinate = atoi(subst.c_str());   

        oldpos = pos; pos=STRING.find(',',oldpos+1); subst = STRING.substr(oldpos+1,pos-oldpos-1);
        pos_count = atof(subst.c_str());

        oldpos = pos; pos=STRING.find(',',oldpos+1); subst = STRING.substr(oldpos+1,pos-oldpos-1);
        neg_count = atof(subst.c_str());    
 
        if (coordinate==prev_coordinate) {
            prev_pos_count =  prev_pos_count + pos_count ;      
            prev_neg_count =  prev_neg_count + neg_count ; } 
            else {
               sba_read_coord[i] = prev_coordinate;   
               sba_read_pos[i]   = prev_pos_count  ;  
               sba_read_neg[i]   = prev_neg_count  ;

             // next step assumes that final line in not "" else you will get missing. If problem use if (STRING=="") break;
               n_sba_reads = n_sba_reads + prev_pos_count + prev_neg_count ;
               prev_coordinate = coordinate; // resetting
               prev_pos_count = pos_count ;
               prev_neg_count = neg_count ;            
               i++; } // end if (coordinate==prev_coordinate)... else

       } // end while(!infile.eof()) 
  
    // final record


     if (coordinate==prev_coordinate)  n_sba_reads = n_sba_reads + prev_pos_count + prev_neg_count;
          
     sba_read_coord[i] = prev_coordinate;   
     sba_read_pos[i]   = prev_pos_count;  
     sba_read_neg[i]   = prev_neg_count;  

  
       n_sba_records = i+1; 


//    cout << " The total number of unique sba read locations " << n_sba_records << "\n";      
//    cout << " The total number of sba reads " << n_sba_reads << "\n";      
// if (aggregated_sba>0)  for(i=0;i<( n_sba_records );i++)      cout << sba_read_coord[i] << "," << sba_read_pos[i] << "," << sba_read_neg[i] << endl;

    infile.close();
   
}     
     
     

void read_multi_reads(const char * filename) {

    string STRING, subst;
	ifstream infile;
	size_t pos, oldpos ;
	int i=0;
    int coordinate=0, prev_coordinate=0 ;
    double pos_count=0.0, neg_count=0.0, prev_pos_count=0.0, prev_neg_count=0.0 ;
    int best_align = 1;
    n_multi_reads =0;

	infile.open (filename);
    if (!(infile).is_open()) { cout << " Read file " << filename << " is not open \n"; exit (1); }
  	
    getline(infile,STRING); 	// skip header 
      	
    getline(infile,STRING);     // read first line
    oldpos = 0; pos=STRING.find(','); subst = STRING.substr(oldpos,pos-oldpos);
    prev_coordinate = atoi(subst.c_str());   

    oldpos = pos; pos=STRING.find(',',oldpos+1); subst = STRING.substr(oldpos+1,pos-oldpos-1);
    prev_pos_count = atof(subst.c_str());

    oldpos = pos; pos=STRING.find(',',oldpos+1); subst = STRING.substr(oldpos+1,pos-oldpos-1);
    prev_neg_count = atof(subst.c_str());    

    oldpos = pos; pos=STRING.find(',',oldpos+1); subst = STRING.substr(oldpos+1,pos-oldpos-1); // skip readnumner and total aliogn
    oldpos = pos; pos=STRING.find(',',oldpos+1); subst = STRING.substr(oldpos+1,pos-oldpos-1);

    oldpos = pos; pos=STRING.find(',',oldpos+1); subst = STRING.substr(oldpos+1,pos-oldpos-1);
    best_align = atoi(subst.c_str());    

    prev_pos_count = prev_pos_count / best_align ;
    prev_neg_count = prev_neg_count / best_align ;	
   	  
	while(!infile.eof()) {            
  
        getline(infile,STRING); 
        if (STRING=="") break;
                
        oldpos = 0; pos=STRING.find(','); subst = STRING.substr(oldpos,pos-oldpos);
        coordinate = atoi(subst.c_str());   

        oldpos = pos; pos=STRING.find(',',oldpos+1); subst = STRING.substr(oldpos+1,pos-oldpos-1);
        pos_count = atof(subst.c_str());

        oldpos = pos; pos=STRING.find(',',oldpos+1); subst = STRING.substr(oldpos+1,pos-oldpos-1);
        neg_count = atof(subst.c_str());    
 
        oldpos = pos; pos=STRING.find(',',oldpos+1); subst = STRING.substr(oldpos+1,pos-oldpos-1); // skip readnumner
        oldpos = pos; pos=STRING.find(',',oldpos+1); subst = STRING.substr(oldpos+1,pos-oldpos-1);
 
        oldpos = pos; pos=STRING.find(',',oldpos+1); subst = STRING.substr(oldpos+1,pos-oldpos-1);
        best_align = atoi(subst.c_str());    

        pos_count = pos_count / best_align ;
        neg_count = neg_count / best_align ;
   
        if (coordinate==prev_coordinate) {
            prev_pos_count =  prev_pos_count + pos_count ;      
            prev_neg_count =  prev_neg_count + neg_count ; } 
            else {
               multi_read_coord[i] = prev_coordinate;   
               multi_read_pos[i]   = prev_pos_count  ;  
               multi_read_neg[i]   = prev_neg_count  ;

             // next step assumes that final line in not "" else you will get missing. If problem use if (STRING=="") break;
               n_multi_reads = n_multi_reads + prev_pos_count + prev_neg_count ;

               prev_coordinate = coordinate; // resetting
               prev_pos_count = pos_count ;
               prev_neg_count = neg_count ;            
               i++; } // end if (coordinate==prev_coordinate)... else

       } // end while(!infile.eof()) 
  
    // final record
  
     if (coordinate==prev_coordinate)  n_multi_reads = n_multi_reads + prev_pos_count + prev_neg_count;
     multi_read_coord[i] = prev_coordinate;   
     multi_read_pos[i]   = prev_pos_count;  
     multi_read_neg[i]   = prev_neg_count;  

  
     n_multi_records = i+1; 
// if (aggregated_multi>0)  for(i=0;i<( n_multi_records );i++)      cout << multi_read_coord[i] << "," << multi_read_pos[i] << "," << multi_read_neg[i] << endl;


    infile.close();
   
}     
